@@ -69,7 +69,7 @@
  *         struct PixaComp
  *
  *   (2) This file has definitions for:
- *         Colors for RGB
+ *         Colors for RGBA
  *         Colors for drawing boxes
  *         Perceptual color weights
  *         Colormap conversion flags
@@ -78,7 +78,7 @@
  *         Sorting flags (by type and direction)
  *         Blending flags
  *         Graphics pixel setting flags
- *         Size filtering flags
+ *         Size and location filter flags
  *         Color component selection flags
  *         16-bit conversion flags
  *         Rotation and shear flags
@@ -89,16 +89,19 @@
  *         Flags for 8 and 16 bit pixel sums
  *         Dithering flags
  *         Distance flags
+ *         Value flags
  *         Statistical measures
  *         Set selection flags
  *         Text orientation flags
  *         Edge orientation flags
  *         Line orientation flags
+ *         Image orientation flags
  *         Scan direction flags
  *         Box size adjustment flags
- *         Flags for selecting box boundaries from two choices
+ *         Flags for modifying box boundaries using a second box
  *         Handling overlapping bounding boxes in boxa
  *         Flags for replacing invalid boxes
+ *         Flags for box corners
  *         Horizontal warp
  *         Pixel selection for resampling
  *         Thinning flags
@@ -109,11 +112,12 @@
  *         Region flags (inclusion, exclusion)
  *         Flags for adding text to a pix
  *         Flags for plotting on a pix
+ *         Flags for making simple masks
  *         Flags for selecting display program
  *         Flags in the 'special' pix field for non-default operations
  *         Handling negative values in conversion to unsigned int
  *         Relative to zero flags
- *         Flags for adding or removing traling slash from string
+ *         Flags for adding or removing trailing slash from string
  *
  *   (3) This file has typedefs for the pix allocator and deallocator functions
  *         alloc_fn()
@@ -174,9 +178,8 @@ struct RGBA_Quad
 typedef struct RGBA_Quad  RGBA_QUAD;
 
 
-
 /*-------------------------------------------------------------------------*
- *                             Colors for 32 bpp                           *
+ *                           Colors for 32 RGBA                            *
  *-------------------------------------------------------------------------*/
 /* <pre>
  *  Notes:
@@ -194,7 +197,7 @@ typedef struct RGBA_Quad  RGBA_QUAD;
  * </pre>
  */
 
-/*! Colors for 32 bpp */
+/*! RGBA Color */
 enum {
     COLOR_RED = 0,        /*!< red color index in RGBA_QUAD    */
     COLOR_GREEN = 1,      /*!< green color index in RGBA_QUAD  */
@@ -215,7 +218,7 @@ static const l_int32  L_ALPHA_SHIFT =
 /*-------------------------------------------------------------------------*
  *                       Colors for drawing boxes                          *
  *-------------------------------------------------------------------------*/
-/*! Colors for drawing boxes */
+/*! Box Color */
 enum {
     L_DRAW_RED = 0,         /*!< draw in red                   */
     L_DRAW_GREEN = 1,       /*!< draw in green                 */
@@ -238,15 +241,15 @@ enum {
  *          International Telecommunications Union, via ITU-R.
  * </pre>
  */
-static const l_float32  L_RED_WEIGHT =   0.3;  /*!< Percept. weight for red   */
-static const l_float32  L_GREEN_WEIGHT = 0.5;  /*!< Percept. weight for green */
-static const l_float32  L_BLUE_WEIGHT =  0.2;  /*!< Percept. weight for blue  */
+static const l_float32 L_RED_WEIGHT =   0.3f; /*!< Percept. weight for red   */
+static const l_float32 L_GREEN_WEIGHT = 0.5f; /*!< Percept. weight for green */
+static const l_float32 L_BLUE_WEIGHT =  0.2f; /*!< Percept. weight for blue  */
 
 
 /*-------------------------------------------------------------------------*
  *                        Flags for colormap conversion                    *
  *-------------------------------------------------------------------------*/
-/*! Flags for colormap conversion */
+/*! Cmap Conversion */
 enum {
     REMOVE_CMAP_TO_BINARY = 0,     /*!< remove colormap for conv to 1 bpp  */
     REMOVE_CMAP_TO_GRAYSCALE = 1,  /*!< remove colormap for conv to 8 bpp  */
@@ -366,7 +369,7 @@ enum {
  *       (4) We use 32 bit pixels for both RGB and RGBA color images.
  *           The A (alpha) byte is ignored in most leptonica functions
  *           operating on color images.  Within each 4 byte pixel, the
- *           colors are ordered from MSB to LSB, as follows:
+ *           color samples are ordered from MSB to LSB, as follows:
  *
  *                |  MSB  |  2nd MSB  |  3rd MSB  |  LSB  |
  *                   red      green       blue      alpha
@@ -395,18 +398,19 @@ enum {
  *                 SET_DATA_BYTE(&pixel, COLOR_BLUE, blueval);
  *                 SET_DATA_BYTE(&pixel, L_ALPHA_CHANNEL, alphaval);
  *
- *           For extra speed we extract these components directly
+ *           More efficiently, these components can be extracted directly
  *           by shifting and masking, explicitly using the values in
  *           L_RED_SHIFT, etc.:
  *                 (pixel32 >> L_RED_SHIFT) & 0xff;         (red)
  *                 (pixel32 >> L_GREEN_SHIFT) & 0xff;       (green)
  *                 (pixel32 >> L_BLUE_SHIFT) & 0xff;        (blue)
  *                 (pixel32 >> L_ALPHA_SHIFT) & 0xff;       (alpha)
- *           All these operations work properly on both big- and little-endians.
+ *           The functions extractRGBValues() and extractRGBAValues() are
+ *           provided to do this.  Likewise, the pixels can be set
+ *           directly by shifting, using composeRGBPixel() and
+ *           composeRGBAPixel().
  *
- *           For a few situations, these color shift values are hard-coded.
- *           Changing the RGB color component ordering through the assignments
- *           in this file will cause functions marked with "***" to fail.
+ *           All these operations work properly on both big- and little-endians.
  *
  *       (5) A reference count is held within each pix, giving the
  *           number of ptrs to the pix.  When a pixClone() call
@@ -481,7 +485,6 @@ struct Box
     l_int32            w;           /*!< box width                         */
     l_int32            h;           /*!< box height                        */
     l_uint32           refcount;    /*!< reference count (1 if no clones)  */
-
 };
 typedef struct Box    BOX;
 
@@ -676,9 +679,13 @@ typedef struct PixaComp PIXAC;
  *               Can be undesirable if very large, such as an image or
  *               an array of images.)
  *     (3) clone (Makes another handle to the same struct, and bumps the
- *                refcount up by 1.  Safe to do unless you're changing
- *                data through one of the handles but don't want those
- *                changes to be seen by the other handle.)
+ *                refcount up by 1.  OK to use except in two situations:
+ *                (a) You change data through one of the handles but don't
+ *                    want those changes to be seen by the other handle.
+ *                (b) The application is multi-threaded.  Because the clone
+ *                    operation is not atomic (e.g., locked with a mutex),
+ *                    it is possible to end up with an incorrect ref count,
+ *                    causing either a memory leak or a crash.
  *
  *  For Pixa and Boxa, which are structs that hold an array of clonable
  *  structs, there is an additional method:
@@ -686,44 +693,51 @@ typedef struct PixaComp PIXAC;
  *                     of 1, but clones all the structs in the array.)
  *
  *  Unlike the other structs, when retrieving a string from an Sarray,
- *  you are allowed to get a handle without a copy or clone (i.e., that
- *  you don't own!).  You must not free or insert such a string!
- *  Specifically, for an Sarray, the copyflag for retrieval is either:
+ *  you are allowed to get a handle without a copy or clone (i.e., the
+ *  string is not owned by the handle).  You must not either free the string
+ *  or insert it in some other struct that would own it.  Specifically,
+ *  for an Sarray, the copyflag for retrieval is either:
  *         L_COPY or L_NOCOPY
  *  and for insertion, the copyflag is either:
  *         L_COPY or one of {L_INSERT , L_NOCOPY} (the latter are equivalent
  *                                                 for insertion))
+ *  Typical patterns are:
+ *  (1) Reference a string in an Sarray with L_NOCOPY and insert a copy
+ *      of it in another Sarray with L_COPY.
+ *  (2) Copy a string from an Sarray with L_COPY and insert it in
+ *      another Sarray with L_INSERT (or L_NOCOPY).
+ *  In both cases, a copy is made and both Sarrays own their instance
+ *  of that string.
  * </pre>
  */
 
-/*! Access and storage flags */
+/*! Object Access */
 enum {
     L_NOCOPY = 0,     /*!< do not copy the object; do not delete the ptr  */
+    L_INSERT = L_NOCOPY,    /*!< stuff it in; do not copy or clone        */
     L_COPY = 1,       /*!< make/use a copy of the object                  */
     L_CLONE = 2,      /*!< make/use clone (ref count) of the object       */
-    L_COPY_CLONE = 3  /*!< make a new object and fill each object in the  */
-                      /*!< array(s) with clones                           */
+    L_COPY_CLONE = 3  /*!< make a new array object (e.g., pixa) and fill  */
+                      /*!< the array with clones (e.g., pix)              */
 };
-static const l_int32  L_INSERT = 0;  /*!< stuff it in; no copy or clone   */
 
 
 /*----------------------------------------------------------------------------*
  *                              Sort flags                                    *
  *----------------------------------------------------------------------------*/
-
-/*! Sort mode flags */
+/*! Sort Mode */
 enum {
     L_SHELL_SORT = 1,            /*!< use shell sort                        */
     L_BIN_SORT = 2               /*!< use bin sort                          */
 };
 
-/*! Sort order flags */
+/*! Sort Order */
 enum {
     L_SORT_INCREASING = 1,       /*!< sort in increasing order              */
     L_SORT_DECREASING = 2        /*!< sort in decreasing order              */
 };
 
-/*! Sort type flags */
+/*! Sort Type */
 enum {
     L_SORT_BY_X = 1,             /*!< sort box or c.c. by left edge location  */
     L_SORT_BY_Y = 2,             /*!< sort box or c.c. by top edge location   */
@@ -742,8 +756,7 @@ enum {
 /*---------------------------------------------------------------------------*
  *                             Blend flags                                   *
  *---------------------------------------------------------------------------*/
-
-/*! Blend flags */
+/*! Blend Types */
 enum {
     L_BLEND_WITH_INVERSE = 1,     /*!< add some of src inverse to itself     */
     L_BLEND_TO_WHITE = 2,         /*!< shift src colors towards white        */
@@ -753,6 +766,7 @@ enum {
                                   /*!< based on blender pix value            */
 };
 
+/*! Paint Selection */
 enum {
     L_PAINT_LIGHT = 1,            /*!< colorize non-black pixels             */
     L_PAINT_DARK = 2              /*!< colorize non-white pixels             */
@@ -762,8 +776,7 @@ enum {
 /*-------------------------------------------------------------------------*
  *                        Graphics pixel setting                           *
  *-------------------------------------------------------------------------*/
-
-/*! Graphics pixel setting */
+/*! Pixel Setting */
 enum {
     L_SET_PIXELS = 1,           /*!< set all bits in each pixel to 1       */
     L_CLEAR_PIXELS = 2,         /*!< set all bits in each pixel to 0       */
@@ -774,20 +787,7 @@ enum {
 /*-------------------------------------------------------------------------*
  *                     Size and location filter flags                      *
  *-------------------------------------------------------------------------*/
-
-/*! Location filter flags */
-enum {
-    L_SELECT_WIDTH = 1,         /*!< width must satisfy constraint         */
-    L_SELECT_HEIGHT = 2,        /*!< height must satisfy constraint        */
-    L_SELECT_XVAL = 3,          /*!< x value satisfy constraint            */
-    L_SELECT_YVAL = 4,          /*!< y value must satisfy constraint       */
-    L_SELECT_IF_EITHER = 5,     /*!< either width or height (or xval       */
-                                /*!< or yval) can satisfy                  */
-    L_SELECT_IF_BOTH = 6        /*!< both width and height (or xval        */
-                                /*!< and yval must satisfy                 */
-};
-
-/*! Size filter flags */
+/*! Size Comparison */
 enum {
     L_SELECT_IF_LT = 1,         /*!< save if value is less than threshold  */
     L_SELECT_IF_GT = 2,         /*!< save if value is more than threshold  */
@@ -795,12 +795,40 @@ enum {
     L_SELECT_IF_GTE = 4         /*!< save if value is >= to the threshold  */
 };
 
+/*! Size Selection */
+enum {
+    L_SELECT_BY_WIDTH = 1,          /*!< select by width; 1 bpp            */
+    L_SELECT_BY_HEIGHT = 2,         /*!< select by height; 1 bpp           */
+    L_SELECT_BY_MAX_DIMENSION = 3,  /*!< select by max of width and        */
+                                    /*!< height; 1 bpp                     */
+    L_SELECT_BY_AREA = 4,           /*!< select by foreground area; 1 bpp  */
+    L_SELECT_BY_PERIMETER = 5       /*!< select by perimeter; 1 bpp        */
+};
+
+/*! Location Filter */
+enum {
+    L_SELECT_WIDTH = 1,         /*!< width must satisfy constraint         */
+    L_SELECT_HEIGHT = 2,        /*!< height must satisfy constraint        */
+    L_SELECT_XVAL = 3,          /*!< x value must satisfy constraint       */
+    L_SELECT_YVAL = 4,          /*!< y value must satisfy constraint       */
+    L_SELECT_IF_EITHER = 5,     /*!< either width or height (or xval       */
+                                /*!< or yval) can satisfy constraint       */
+    L_SELECT_IF_BOTH = 6        /*!< both width and height (or xval        */
+                                /*!< and yval must satisfy constraint      */
+};
+
+/*! Boxa Check */
+enum {
+    L_CHECK_WIDTH = 1,          /*!< check and possibly modify width       */
+    L_CHECK_HEIGHT = 2,         /*!< check and possibly modify height      */
+    L_CHECK_BOTH = 3            /*!< check and possibly modify both        */
+};
+
 
 /*-------------------------------------------------------------------------*
  *                    Color component selection flags                      *
  *-------------------------------------------------------------------------*/
-
-/*! Color component selection flags */
+/*! Color Selection */
 enum {
     L_SELECT_RED = 1,           /*!< use red component                     */
     L_SELECT_GREEN = 2,         /*!< use green component                   */
@@ -816,8 +844,7 @@ enum {
 /*-------------------------------------------------------------------------*
  *                         16-bit conversion flags                         *
  *-------------------------------------------------------------------------*/
-
-/*! 16-bit conversion flags */
+/*! 16-bit Conversion */
 enum {
     L_LS_BYTE = 1,              /*!< use LSB                               */
     L_MS_BYTE = 2,              /*!< use MSB                               */
@@ -832,21 +859,20 @@ enum {
 /*-------------------------------------------------------------------------*
  *                        Rotate and shear flags                           *
  *-------------------------------------------------------------------------*/
-
-/*! Rotate flags */
+/*! Rotation Type */
 enum {
     L_ROTATE_AREA_MAP = 1,     /*!< use area map rotation, if possible     */
     L_ROTATE_SHEAR = 2,        /*!< use shear rotation                     */
     L_ROTATE_SAMPLING = 3      /*!< use sampling                           */
 };
 
-/*! Background flags */
+/*! Background Color */
 enum {
     L_BRING_IN_WHITE = 1,      /*!< bring in white pixels from the outside */
     L_BRING_IN_BLACK = 2       /*!< bring in black pixels from the outside */
 };
 
-/*! Shear flags */
+/*! Shear Point */
 enum {
     L_SHEAR_ABOUT_CORNER = 1,  /*!< shear image about UL corner            */
     L_SHEAR_ABOUT_CENTER = 2   /*!< shear image about center               */
@@ -856,8 +882,7 @@ enum {
 /*-------------------------------------------------------------------------*
  *                     Affine transform order flags                        *
  *-------------------------------------------------------------------------*/
-
-/*! Affine transform order flags */
+/*! Affine Transform Order */
 enum {
     L_TR_SC_RO = 1,            /*!< translate, scale, rotate               */
     L_SC_RO_TR = 2,            /*!< scale, rotate, translate               */
@@ -871,8 +896,7 @@ enum {
 /*-------------------------------------------------------------------------*
  *                       Grayscale filling flags                           *
  *-------------------------------------------------------------------------*/
-
-/*! Grayscale filling flags */
+/*! Grayscale Fill */
 enum {
     L_FILL_WHITE = 1,         /*!< fill white pixels (e.g, in fg map)      */
     L_FILL_BLACK = 2          /*!< fill black pixels (e.g., in bg map)     */
@@ -882,8 +906,7 @@ enum {
 /*-------------------------------------------------------------------------*
  *                   Flags for setting to white or black                   *
  *-------------------------------------------------------------------------*/
-
-/*! Flags for setting to white or black */
+/*! BlackWhite Set */
 enum {
     L_SET_WHITE = 1,         /*!< set pixels to white                      */
     L_SET_BLACK = 2          /*!< set pixels to black                      */
@@ -893,8 +916,7 @@ enum {
 /*-------------------------------------------------------------------------*
  *                  Flags for getting white or black value                 *
  *-------------------------------------------------------------------------*/
-
-/*! Flags for getting white or black value */
+/*! BlackWhite Get */
 enum {
     L_GET_WHITE_VAL = 1,     /*!< get white pixel value                    */
     L_GET_BLACK_VAL = 2      /*!< get black pixel value                    */
@@ -904,8 +926,7 @@ enum {
 /*-------------------------------------------------------------------------*
  *                  Flags for 8 bit and 16 bit pixel sums                  *
  *-------------------------------------------------------------------------*/
-
-/*! Flags for 8 bit and 16 bit pixel sums */
+/*! BlackWhite Sum */
 enum {
     L_WHITE_IS_MAX = 1, /*!< white pixels are 0xff or 0xffff; black are 0  */
     L_BLACK_IS_MAX = 2  /*!< black pixels are 0xff or 0xffff; white are 0  */
@@ -917,8 +938,7 @@ enum {
  *         If within this grayscale distance from black or white,          *
  *         do not propagate excess or deficit to neighboring pixels.       *
  *-------------------------------------------------------------------------*/
-
-/*! Dither parameters */
+/*! Dither Distance */
 enum {
     DEFAULT_CLIP_LOWER_1 = 10, /*!< dist to black with no prop; 1 bpp      */
     DEFAULT_CLIP_UPPER_1 = 10, /*!< dist to black with no prop; 1 bpp      */
@@ -928,21 +948,33 @@ enum {
 
 
 /*-------------------------------------------------------------------------*
- *                             Distance flags                              *
+ *                          Distance type flags                            *
  *-------------------------------------------------------------------------*/
-
-/*! Distance flags */
+/*! Distance Type */
 enum {
     L_MANHATTAN_DISTANCE = 1,  /*!< L1 distance (e.g., in color space)     */
     L_EUCLIDEAN_DISTANCE = 2   /*!< L2 distance                            */
 };
 
 
+/*-------------------------------------------------------------------------*
+ *                         Distance Value flags                            *
+ *-------------------------------------------------------------------------*/
+/*! Distance Value */
+enum {
+    L_NEGATIVE = 1,      /*!< values < 0                                   */
+    L_NON_NEGATIVE = 2,  /*!< values >= 0                                  */
+    L_POSITIVE = 3,      /*!< values > 0                                   */
+    L_NON_POSITIVE = 4,  /*!< values <= 0                                  */
+    L_ZERO = 5,          /*!< values = 0                                   */
+    L_ALL = 6            /*!< all values                                   */
+};
+
+
 /*-------------------------------------------------------------------------*
  *                         Statistical measures                            *
  *-------------------------------------------------------------------------*/
-
-/*! Statistical measures */
+/*! Stats Type */
 enum {
     L_MEAN_ABSVAL = 1,         /*!< average of abs values                  */
     L_MEDIAN_VAL = 2,          /*!< median value of set                    */
@@ -955,10 +987,9 @@ enum {
 
 
 /*-------------------------------------------------------------------------*
- *                          Set selection flags                            *
+ *                       Set index selection flags                         *
  *-------------------------------------------------------------------------*/
-
-/*! Set selection flags */
+/*! Index Selection */
 enum {
     L_CHOOSE_CONSECUTIVE = 1,  /*!< select 'n' consecutive                 */
     L_CHOOSE_SKIP_BY = 2       /*!< select at intervals of 'n'             */
@@ -968,8 +999,7 @@ enum {
 /*-------------------------------------------------------------------------*
  *                         Text orientation flags                          *
  *-------------------------------------------------------------------------*/
-
-/*! Text orientation flags */
+/*! Text Orientation */
 enum {
     L_TEXT_ORIENT_UNKNOWN = 0, /*!< low confidence on text orientation     */
     L_TEXT_ORIENT_UP = 1,      /*!< portrait, text rightside-up            */
@@ -982,8 +1012,7 @@ enum {
 /*-------------------------------------------------------------------------*
  *                         Edge orientation flags                          *
  *-------------------------------------------------------------------------*/
-
-/*! Edge orientation flags */
+/*! Edge Orientation */
 enum {
     L_HORIZONTAL_EDGES = 0,   /*!< filters for horizontal edges            */
     L_VERTICAL_EDGES = 1,     /*!< filters for vertical edges              */
@@ -994,8 +1023,7 @@ enum {
 /*-------------------------------------------------------------------------*
  *                         Line orientation flags                          *
  *-------------------------------------------------------------------------*/
-
-/*! Line orientation flags */
+/*! Line Orientation */
 enum {
     L_HORIZONTAL_LINE = 0,   /*!< horizontal line                          */
     L_POS_SLOPE_LINE = 1,    /*!< 45 degree line with positive slope       */
@@ -1005,11 +1033,20 @@ enum {
 };
 
 
+/*-------------------------------------------------------------------------*
+ *                         Image orientation flags                         *
+ *-------------------------------------------------------------------------*/
+/*! Image Orientation */
+enum {
+    L_PORTRAIT_MODE = 0,   /*!< typical: page is viewed with height > width  */
+    L_LANDSCAPE_MODE = 1   /*!< page is viewed at 90 deg to portrait mode    */
+};
+
+
 /*-------------------------------------------------------------------------*
  *                           Scan direction flags                          *
  *-------------------------------------------------------------------------*/
-
-/*! Scan direction flags */
+/*! Scan Direction */
 enum {
     L_FROM_LEFT = 0,         /*!< scan from left                           */
     L_FROM_RIGHT = 1,        /*!< scan from right                          */
@@ -1026,8 +1063,7 @@ enum {
 /*-------------------------------------------------------------------------*
  *                Box size adjustment and location flags                   *
  *-------------------------------------------------------------------------*/
-
-/*! Box size adjustment and location flags */
+/*! Box Adjustment */
 enum {
     L_ADJUST_SKIP = 0,           /*!< do not adjust                        */
     L_ADJUST_LEFT = 1,           /*!< adjust left edge                     */
@@ -1050,49 +1086,61 @@ enum {
 
 
 /*-------------------------------------------------------------------------*
- *          Flags for selecting box boundaries from two choices            *
+ *          Flags for modifying box boundaries using a second box          *
  *-------------------------------------------------------------------------*/
-
-/*! Flags for selecting box boundaries from two choices */
+/*! Box Boundary Mod */
 enum {
     L_USE_MINSIZE = 1,           /*!< use boundaries giving min size       */
     L_USE_MAXSIZE = 2,           /*!< use boundaries giving max size       */
-    L_SUB_ON_BIG_DIFF = 3,       /*!< substitute boundary if big abs diff  */
-    L_USE_CAPPED_MIN = 4,        /*!< substitute boundary with capped min  */
-    L_USE_CAPPED_MAX = 5         /*!< substitute boundary with capped max  */
+    L_SUB_ON_LOC_DIFF = 3,       /*!< modify boundary if big location diff */
+    L_SUB_ON_SIZE_DIFF = 4,      /*!< modify boundary if big size diff     */
+    L_USE_CAPPED_MIN = 5,        /*!< modify boundary with capped min      */
+    L_USE_CAPPED_MAX = 6         /*!< modify boundary with capped max      */
 };
 
+
 /*-------------------------------------------------------------------------*
  *              Handling overlapping bounding boxes in boxa                *
  *-------------------------------------------------------------------------*/
-
-/*! Handling overlapping bounding boxes in Boxa */
+/*! Box Overlap Mod */
 enum {
     L_COMBINE = 1,         /*!< resize to bounding region; remove smaller  */
     L_REMOVE_SMALL = 2     /*!< only remove smaller                        */
 };
 
+
 /*-------------------------------------------------------------------------*
  *                    Flags for replacing invalid boxes                    *
  *-------------------------------------------------------------------------*/
-
-/*! Flags for replacing invalid boxes */
+/*! Box Replacement */
 enum {
     L_USE_ALL_BOXES = 1,         /*!< consider all boxes in the sequence   */
     L_USE_SAME_PARITY_BOXES = 2  /*!< consider boxes with the same parity  */
 };
 
+
+/*-------------------------------------------------------------------------*
+ *                          Flags for box corners                          *
+ *-------------------------------------------------------------------------*/
+/*! Box Corners */
+enum {
+    L_UPPER_LEFT = 1,         /*!< UL corner                               */
+    L_UPPER_RIGHT = 2,        /*!< UR corner                               */
+    L_LOWER_LEFT = 3,         /*!< LL corner                               */
+    L_LOWER_RIGHT = 4         /*!< LR corner                               */
+};
+
+
 /*-------------------------------------------------------------------------*
  *                            Horizontal warp                              *
  *-------------------------------------------------------------------------*/
-
-/*! Horizonal warp direction */
+/*! Horiz Warp Stretch */
 enum {
     L_WARP_TO_LEFT = 1,    /*!< increasing stretch or contraction to left  */
     L_WARP_TO_RIGHT = 2    /*!< increasing stretch or contraction to right */
 };
 
-/*! Horizonal warp stretch mode */
+/*! Horiz Warp Mode */
 enum {
     L_LINEAR_WARP = 1,     /*!< stretch or contraction grows linearly      */
     L_QUADRATIC_WARP = 2   /*!< stretch or contraction grows quadratically */
@@ -1102,8 +1150,7 @@ enum {
 /*-------------------------------------------------------------------------*
  *                      Pixel selection for resampling                     *
  *-------------------------------------------------------------------------*/
-
-/*! Pixel selection for resampling */
+/*! Pixel Selection */
 enum {
     L_INTERPOLATED = 1,    /*!< linear interpolation from src pixels       */
     L_SAMPLED = 2          /*!< nearest src pixel sampling only            */
@@ -1113,8 +1160,7 @@ enum {
 /*-------------------------------------------------------------------------*
  *                             Thinning flags                              *
  *-------------------------------------------------------------------------*/
-
-/*! Thinning flags */
+/*! Thinning Polarity */
 enum {
     L_THIN_FG = 1,             /*!< thin foreground of 1 bpp image         */
     L_THIN_BG = 2              /*!< thin background of 1 bpp image         */
@@ -1124,8 +1170,7 @@ enum {
 /*-------------------------------------------------------------------------*
  *                            Runlength flags                              *
  *-------------------------------------------------------------------------*/
-
-/*! Runlength flags */
+/*! Runlength Direction */
 enum {
     L_HORIZONTAL_RUNS = 0,   /*!< determine runlengths of horizontal runs  */
     L_VERTICAL_RUNS = 1      /*!< determine runlengths of vertical runs    */
@@ -1135,8 +1180,7 @@ enum {
 /*-------------------------------------------------------------------------*
  *                          Edge filter flags                              *
  *-------------------------------------------------------------------------*/
-
-/*! Edge filter flags */
+/*! Edge Filter */
 enum {
     L_SOBEL_EDGE = 1,        /*!< Sobel edge filter                        */
     L_TWO_SIDED_EDGE = 2     /*!< Two-sided edge filter                    */
@@ -1146,8 +1190,7 @@ enum {
 /*-------------------------------------------------------------------------*
  *             Subpixel color component ordering in LCD display            *
  *-------------------------------------------------------------------------*/
-
-/*! Subpixel color component ordering in LC display */
+/*! Subpixel Color Order */
 enum {
     L_SUBPIXEL_ORDER_RGB = 1,   /*!< sensor order left-to-right RGB        */
     L_SUBPIXEL_ORDER_BGR = 2,   /*!< sensor order left-to-right BGR        */
@@ -1159,8 +1202,7 @@ enum {
 /*-------------------------------------------------------------------------*
  *                          HSV histogram flags                            *
  *-------------------------------------------------------------------------*/
-
-/*! HSV histogram flags */
+/*! HSV Histogram */
 enum {
     L_HS_HISTO = 1,            /*!< Use hue-saturation histogram           */
     L_HV_HISTO = 2,            /*!< Use hue-value histogram                */
@@ -1169,21 +1211,19 @@ enum {
 
 
 /*-------------------------------------------------------------------------*
- *                    Region flags (inclusion, exclusion)                  *
+ *                HSV Region flags (inclusion, exclusion)                  *
  *-------------------------------------------------------------------------*/
-
-/*! Region flags (inclusion, exclusion) */
+/*! HSV Region */
 enum {
-    L_INCLUDE_REGION = 1,      /*!< Use hue-saturation histogram           */
-    L_EXCLUDE_REGION = 2       /*!< Use hue-value histogram                */
+    L_INCLUDE_REGION = 1,      /*!< Use pixels with specified HSV region   */
+    L_EXCLUDE_REGION = 2       /*!< Use pixels outside HSV region          */
 };
 
 
 /*-------------------------------------------------------------------------*
- *                    Flags for adding text to a pix                       *
+ *                Location flags for adding text to a pix                  *
  *-------------------------------------------------------------------------*/
-
-/*! Flags for adding text to a Pix */
+/*! Add Text Location */
 enum {
     L_ADD_ABOVE = 1,           /*!< Add text above the image               */
     L_ADD_BELOW = 2,           /*!< Add text below the image               */
@@ -1199,8 +1239,7 @@ enum {
 /*-------------------------------------------------------------------------*
  *                       Flags for plotting on a pix                       *
  *-------------------------------------------------------------------------*/
-
-/*! Flags for plotting on a Pix */
+/*! Pix Plot */
 enum {
     L_PLOT_AT_TOP = 1,         /*!< Plot horizontally at top               */
     L_PLOT_AT_MID_HORIZ = 2,   /*!< Plot horizontally at middle            */
@@ -1211,11 +1250,20 @@ enum {
 };
 
 
+/*-------------------------------------------------------------------------*
+ *                    Flags for making simple masks                        *
+ *-------------------------------------------------------------------------*/
+/*! Mask Generation */
+enum {
+    L_USE_INNER = 1,           /*!< Select the interior part               */
+    L_USE_OUTER = 2            /*!< Select the outer part (e.g., a frame)  */
+};
+
+
 /*-------------------------------------------------------------------------*
  *                   Flags for selecting display program                   *
  *-------------------------------------------------------------------------*/
-
-/*! Flags for selecting display program */
+/*! Display Program */
 enum {
     L_DISPLAY_WITH_XZGV = 1,  /*!< Use xzgv with pixDisplay()              */
     L_DISPLAY_WITH_XLI = 2,   /*!< Use xli with pixDisplay()               */
@@ -1230,7 +1278,6 @@ enum {
  *      - 10-19 are used for zlib compression in png write                 *
  *      - 4 and 8 are used for specifying connectivity in labelling        *
  *-------------------------------------------------------------------------*/
-
 /*! Flags used in Pix::special */
 enum {
     L_NO_CHROMA_SAMPLING_JPEG = 1   /*!< Write full resolution chroma      */
@@ -1240,8 +1287,7 @@ enum {
 /*-------------------------------------------------------------------------*
  *          Handling negative values in conversion to unsigned int         *
  *-------------------------------------------------------------------------*/
-
-/*! Handling negative values in conversion to unsigned int */
+/*! Negative Value */
 enum {
     L_CLIP_TO_ZERO = 1,      /*!< Clip negative values to 0                */
     L_TAKE_ABSVAL = 2        /*!< Convert to positive using L_ABS()        */
@@ -1251,8 +1297,7 @@ enum {
 /*-------------------------------------------------------------------------*
  *                        Relative to zero flags                           *
  *-------------------------------------------------------------------------*/
-
-/*! Relative to zero flags */
+/*! Relative To Zero */
 enum {
     L_LESS_THAN_ZERO = 1,    /*!< Choose values less than zero             */
     L_EQUAL_TO_ZERO = 2,     /*!< Choose values equal to zero              */
@@ -1261,10 +1306,9 @@ enum {
 
 
 /*-------------------------------------------------------------------------*
- *         Flags for adding or removing traling slash from string          *
+ *         Flags for adding or removing trailing slash from string         *
  *-------------------------------------------------------------------------*/
-
-/*! Flags for adding or removing traling slash from string */
+/*! Trailing Slash */
 enum {
     L_ADD_TRAIL_SLASH = 1,     /*!< Add trailing slash to string           */
     L_REMOVE_TRAIL_SLASH = 2   /*!< Remove trailing slash from string      */
